@@ -166,7 +166,9 @@ constexpr TagDetails pentaxModel[] = {
     {0x13254, "K-3 Mark III"},
     {0x13290, "WG-70"},
     {0x1329a, "GR IIIx"},
+    {0x132b8, "KF"},
     {0x132d6, "K-3 Mark III Monochrome"},
+    {0x132e0, "GR IV"},
 };
 
 //! Quality, tag 0x0008
