@@ -5,13 +5,14 @@ import system_tests
 
 class TestCvePoC(metaclass=system_tests.CaseMeta):
 
-    url = "https://github.com/Exiv2/exiv2/issues/53"
+    url = "https://github.com/Exiv2/exiv2/issues/55"
 
-    filename = "$data_path/POC6"
+    filename = "$data_path/POC8"
     commands = ["$exiv2 $filename"]
-    stdout = ["""File name       : $filename
-File size       : 60 Bytes
-MIME type       : image/x-olympus-orf
+    stdout = [
+        """File name       : $filename
+File size       : 2044 Bytes
+MIME type       : image/tiff
 Image size      : 0 x 0
 Thumbnail       : None
 Camera make     : 
@@ -34,11 +35,9 @@ White balance   :
 Copyright       : 
 Exif comment    : 
 
-"""]
-    stderr = [
-        """Error: Directory Image, entry 0x0000 has invalid size 4294967295*1; skipping entry.
-Error: Offset of directory Image, entry 0x0000 is out of bounds: Offset = 0x7e000000; truncating the entry
-Error: Offset of directory Image, entry 0x0111 is out of bounds: Offset = 0x7e000000; truncating the entry
 """
     ]
+    stderr = ["ignored"]
     retval = [0]
+
+    compare_stderr = system_tests.check_no_ASAN_UBSAN_errors
