@@ -1,5 +1,5 @@
 // =================================================================================================
-// Copyright 2002-2007 Adobe Systems Incorporated
+// Copyright 2002-2008 Adobe Systems Incorporated
 // All Rights Reserved.
 //
 // NOTICE:  Adobe permits you to use, modify, and distribute this file in accordance with the terms
@@ -12,15 +12,16 @@
 #include "XMP_Environment.h"	// ! This must be the first include!
 #include "XMPCore_Impl.hpp"
 
+#include <stdio.h>
+
 #include "XMPMeta.hpp"
 #include "XMPIterator.hpp"
 #include "XMPUtils.hpp"
-
 #include "XMP_Version.h"
 #include "UnicodeInlines.incl_cpp"
 #include "UnicodeConversions.hpp"
 
-#include <cstdio>
+#include <algorithm>	// For sort and stable_sort.
 
 #if XMP_DebugBuild
 	#include <iostream>
@@ -29,11 +30,13 @@
 using namespace std;
 
 #if XMP_WinBuild
+#ifdef _MSC_VER
 	#pragma warning ( disable : 4533 )	// initialization of '...' is skipped by 'goto ...'
 	#pragma warning ( disable : 4702 )	// unreachable code
 	#pragma warning ( disable : 4800 )	// forcing value to bool 'true' or 'false' (performance warning)
 	#pragma warning ( disable : 4996 )	// '...' was declared deprecated
 #endif
+#endif
 
 
 // *** Use the XMP_PropIsXyz (Schema, Simple, Struct, Array, ...) macros
@@ -80,14 +83,73 @@ static const char * kTenSpaces = "          ";
 
 #define OutProcULong(num)	{ snprintf ( buffer, sizeof(buffer), "%lu", (num) ); /* AUDIT: Using sizeof for snprintf length is safe */ \
 							  status = (*outProc) ( refCon, buffer, strlen(buffer) );  if ( status != 0 ) goto EXIT; }
-
+#ifdef __APPLE__
+#define OutProcHexInt(num)	{ snprintf ( buffer, sizeof(buffer), "%X", (num) ); /* AUDIT: Using sizeof for snprintf length is safe */	\
+							  status = (*outProc) ( refCon, buffer, strlen(buffer) );  if ( status != 0 ) goto EXIT; }
+#else
 #define OutProcHexInt(num)	{ snprintf ( buffer, sizeof(buffer), "%lX", (num) ); /* AUDIT: Using sizeof for snprintf length is safe */	\
 							  status = (*outProc) ( refCon, buffer, strlen(buffer) );  if ( status != 0 ) goto EXIT; }
+#endif
+
+#define OutProcHexByte(num)	{ snprintf ( buffer, sizeof(buffer), "%.2X", (num) ); /* AUDIT: Using sizeof for snprintf length is safe */	\
+							  status = (*outProc) ( refCon, buffer, strlen(buffer) );  if ( status != 0 ) goto EXIT; }
 
 static const char * kIndent = "   ";
 #define OutProcIndent(lev)	{ for ( size_t i = 0; i < (lev); ++i ) OutProcNChars ( kIndent, 3 ); }
 
 
+// -------------------------------------------------------------------------------------------------
+// DumpClearString
+// ---------------
+
+static XMP_Status
+DumpClearString ( const XMP_VarString & value, XMP_TextOutputProc outProc, void * refCon )
+{
+
+	char buffer [20];
+	bool prevNormal;
+	XMP_Status status = 0;
+	
+	XMP_StringPtr spanStart, spanEnd;
+	XMP_StringPtr valueEnd = &value[0] + value.size();
+	
+	spanStart = &value[0];
+	while ( spanStart < valueEnd ) {
+	
+		// Output the next span of regular characters.
+		for ( spanEnd = spanStart; spanEnd < valueEnd; ++spanEnd ) {
+			if ( (unsigned char)(*spanEnd) > 0x7F ) break;
+			if ( (*spanEnd < 0x20) && (*spanEnd != kTab) && (*spanEnd != kLF) ) break;
+		}
+		if ( spanStart != spanEnd ) status = (*outProc) ( refCon,  spanStart, (spanEnd-spanStart) );
+		if ( status != 0 ) break;
+		spanStart = spanEnd;
+		
+		// Output the next span of irregular characters.
+		prevNormal = true;
+		for ( spanEnd = spanStart; spanEnd < valueEnd; ++spanEnd ) {
+			if ( ((0x20 <= *spanEnd) && ((unsigned char)(*spanEnd) <= 0x7F)) || (*spanEnd == kTab) || (*spanEnd == kLF) ) break;
+			char space = ' ';
+			if ( prevNormal ) space = '<';
+			status = (*outProc) ( refCon, &space, 1 );
+			if ( status != 0 ) break;
+			OutProcHexByte ( *spanEnd );
+			prevNormal = false;
+		}
+		if ( ! prevNormal ) {
+			status = (*outProc) ( refCon, ">", 1 );
+			if ( status != 0 ) return status;
+		}
+		spanStart = spanEnd;
+
+	}
+
+EXIT:	
+	return status;
+	
+}	// DumpClearString
+
+
 // -------------------------------------------------------------------------------------------------
 // DumpStringMap
 // -------------
@@ -111,10 +173,10 @@ DumpStringMap ( const XMP_StringMap & map, XMP_StringPtr label, XMP_TextOutputPr
 	
 	for ( currPos = map.begin(); currPos != endPos; ++currPos ) {
 		OutProcNChars ( "  ", 2 );
-		OutProcString ( currPos->first );
+		DumpClearString ( currPos->first, outProc, refCon );
 		OutProcPadding ( maxLen - currPos->first.size() );
 		OutProcNChars ( " => ", 4 );
-		OutProcString ( currPos->second );
+		DumpClearString ( currPos->second, outProc, refCon );
 		OutProcNewline();
 	}
 	
@@ -213,7 +275,7 @@ DumpPropertyTree ( const XMP_Node *	  currNode,
 	OutProcIndent ( (size_t)indent );
 	if ( itemIndex == 0 ) {
 		if ( currNode->options & kXMP_PropIsQualifier ) OutProcNChars ( "? ", 2 );
-		OutProcString ( currNode->name );
+		DumpClearString ( currNode->name, outProc, refCon );
 	} else {
 		OutProcNChars ( "[", 1 );
 		OutProcULong ( static_cast<unsigned long>(itemIndex) );
@@ -222,7 +284,7 @@ DumpPropertyTree ( const XMP_Node *	  currNode,
 
 	if ( ! (currNode->options & kXMP_PropCompositeMask) ) {
 		OutProcNChars ( " = \"", 4 );
-		OutProcString ( currNode->value );
+		DumpClearString ( currNode->value, outProc, refCon );
 		OutProcNChars ( "\"", 1 );
 	}
 
@@ -410,6 +472,114 @@ static void DumpXMLTree ( FILE * log, const XML_Node & node, int indent )
 #endif	// DumpXMLParseTree
 
 
+// -------------------------------------------------------------------------------------------------
+// CompareNodeNames
+// ----------------
+//
+// Comparison routine for sorting XMP nodes by name. The name "xml:lang" is less than anything else,
+// and "rdf:type" is less than anything except "xml:lang". This preserves special rules for qualifiers.
+
+static bool
+CompareNodeNames ( XMP_Node * left, XMP_Node * right )
+{
+
+	if ( left->name  == "xml:lang" ) return true;
+	if ( right->name == "xml:lang" ) return false;
+	
+	if ( left->name  == "rdf:type" ) return true;
+	if ( right->name == "rdf:type" ) return false;
+	
+	return ( left->name < right->name );
+	
+}	// CompareNodeNames
+
+
+// -------------------------------------------------------------------------------------------------
+// CompareNodeValues
+// -----------------
+//
+// Comparison routine for sorting XMP nodes by value.
+
+static bool
+CompareNodeValues ( XMP_Node * left, XMP_Node * right )
+{
+
+	if ( XMP_PropIsSimple ( left->options ) && XMP_PropIsSimple ( right->options ) ) {
+		return ( left->value < right->value );
+	}
+
+	XMP_OptionBits leftForm  = left->options & kXMP_PropCompositeMask;
+	XMP_OptionBits rightForm = right->options & kXMP_PropCompositeMask;
+	
+	return ( leftForm < rightForm );
+	
+}	// CompareNodeValues
+
+
+// -------------------------------------------------------------------------------------------------
+// CompareNodeLangs
+// ----------------
+//
+// Comparison routine for sorting XMP nodes by xml:lang qualifier. An "x-default" value is less than
+// any other language.
+
+static bool
+CompareNodeLangs ( XMP_Node * left, XMP_Node * right )
+{
+
+	if ( left->qualifiers.empty()  || (left->qualifiers[0]->name  != "xml:lang") ) return false;
+	if ( right->qualifiers.empty() || (right->qualifiers[0]->name != "xml:lang") ) return false;
+	
+	if ( left->qualifiers[0]->value  == "x-default" ) return true;
+	if ( right->qualifiers[0]->value == "x-default" ) return false;
+	
+	return ( left->qualifiers[0]->value < right->qualifiers[0]->value );
+	
+}	// CompareNodeLangs
+
+
+// -------------------------------------------------------------------------------------------------
+// SortWithinOffspring
+// -------------------
+//
+// Sort one level down, within the elements of a node vector. This sorts the qualifiers of each
+// node. If the node is a struct it sorts the fields by names. If the node is an unordered array it
+// sorts the elements by value. If the node is an AltText array it sorts the elements by language.
+
+static void
+SortWithinOffspring ( XMP_NodeOffspring & nodeVec )
+{
+
+	for ( size_t i = 0, limit = nodeVec.size(); i < limit; ++i ) {
+	
+		XMP_Node * currPos = nodeVec[i];
+	
+		if ( ! currPos->qualifiers.empty() ) {
+			sort ( currPos->qualifiers.begin(), currPos->qualifiers.end(), CompareNodeNames );
+			SortWithinOffspring ( currPos->qualifiers );
+		}
+		
+		if ( ! currPos->children.empty() ) {
+
+			if ( XMP_PropIsStruct ( currPos->options ) || XMP_NodeIsSchema ( currPos->options ) ) {
+				sort ( currPos->children.begin(), currPos->children.end(), CompareNodeNames );
+			} else if ( XMP_PropIsArray ( currPos->options ) ) {
+				if ( XMP_ArrayIsUnordered ( currPos->options ) ) {
+					stable_sort ( currPos->children.begin(), currPos->children.end(), CompareNodeValues );
+				} else if ( XMP_ArrayIsAltText ( currPos->options ) ) {
+					sort ( currPos->children.begin(), currPos->children.end(), CompareNodeLangs );
+				}
+			}
+
+			SortWithinOffspring ( currPos->children );
+			
+		}
+
+	}
+	
+}	// SortWithinOffspring
+
+
 // =================================================================================================
 // Constructors
 // ============
@@ -453,7 +623,8 @@ XMPMeta::~XMPMeta() RELEASE_NO_THROW
 XMPMeta::GetVersionInfo ( XMP_VersionInfo * info )
 {
 
-	memset ( info, 0, sizeof(XMP_VersionInfo) );
+	memset ( info, 0, sizeof(*info) );	// AUDIT: Safe, using sizeof the destination.
+	XMP_Assert ( sizeof(*info) == sizeof(XMP_VersionInfo) );
 	
 	info->major   = XMP_API_VERSION_MAJOR;
 	info->minor   = XMP_API_VERSION_MINOR;
@@ -495,65 +666,63 @@ XMPMeta::Initialize()
 	InitializeUnicodeConversions();
 	
 	// Register standard namespaces and aliases.
+	RegisterNamespace ( kXMP_NS_XML, "xml" );
+	RegisterNamespace ( kXMP_NS_RDF, "rdf" );
+	RegisterNamespace ( kXMP_NS_DC, "dc" );
+
+	RegisterNamespace ( kXMP_NS_XMP, "xmp" );
+	RegisterNamespace ( kXMP_NS_PDF, "pdf" );
+	RegisterNamespace ( kXMP_NS_Photoshop, "photoshop" );
+	RegisterNamespace ( kXMP_NS_PSAlbum, "album" );
+	RegisterNamespace ( kXMP_NS_EXIF, "exif" );
+	RegisterNamespace ( kXMP_NS_EXIF_Aux, "aux" );
+	RegisterNamespace ( kXMP_NS_TIFF, "tiff" );
+	RegisterNamespace ( kXMP_NS_PNG, "png" );
+	RegisterNamespace ( kXMP_NS_JPEG, "jpeg" );
+	RegisterNamespace ( kXMP_NS_JP2K, "jp2k" );
+	RegisterNamespace ( kXMP_NS_CameraRaw, "crs" );
+	RegisterNamespace ( kXMP_NS_ASF, "asf" );
+	RegisterNamespace ( kXMP_NS_WAV, "wav" );
+
+	RegisterNamespace ( kXMP_NS_AdobeStockPhoto, "bmsp" );
+	RegisterNamespace ( kXMP_NS_CreatorAtom, "creatorAtom" );
+
+	RegisterNamespace ( kXMP_NS_XMP_Rights, "xmpRights" );
+	RegisterNamespace ( kXMP_NS_XMP_MM, "xmpMM" );
+	RegisterNamespace ( kXMP_NS_XMP_BJ, "xmpBJ" );
+	RegisterNamespace ( kXMP_NS_XMP_Note, "xmpNote" );
+
+	RegisterNamespace ( kXMP_NS_DM, "xmpDM" );
+	RegisterNamespace ( kXMP_NS_XMP_Text, "xmpT" );
+	RegisterNamespace ( kXMP_NS_XMP_PagedFile, "xmpTPg" );
+	RegisterNamespace ( kXMP_NS_XMP_Graphics, "xmpG" );
+	RegisterNamespace ( kXMP_NS_XMP_Image, "xmpGImg" );
+
+	RegisterNamespace ( kXMP_NS_XMP_Font, "stFnt" );
+	RegisterNamespace ( kXMP_NS_XMP_Dimensions, "stDim" );
+	RegisterNamespace ( kXMP_NS_XMP_ResourceEvent, "stEvt" );
+	RegisterNamespace ( kXMP_NS_XMP_ResourceRef, "stRef" );
+	RegisterNamespace ( kXMP_NS_XMP_ST_Version, "stVer" );
+	RegisterNamespace ( kXMP_NS_XMP_ST_Job, "stJob" );
+	RegisterNamespace ( kXMP_NS_XMP_ManifestItem, "stMfs" );
+
+	RegisterNamespace ( kXMP_NS_XMP_IdentifierQual, "xmpidq" );
+
+	RegisterNamespace ( kXMP_NS_IPTCCore, "Iptc4xmpCore" );
+	RegisterNamespace ( kXMP_NS_DICOM, "DICOM" );
+
+	RegisterNamespace ( kXMP_NS_PDFA_Schema, "pdfaSchema" );
+	RegisterNamespace ( kXMP_NS_PDFA_Property, "pdfaProperty" );
+	RegisterNamespace ( kXMP_NS_PDFA_Type, "pdfaType" );
+	RegisterNamespace ( kXMP_NS_PDFA_Field, "pdfaField" );
+	RegisterNamespace ( kXMP_NS_PDFA_ID, "pdfaid" );
+	RegisterNamespace ( kXMP_NS_PDFA_Extension, "pdfaExtension" );
+
+	RegisterNamespace ( kXMP_NS_PDFX, "pdfx" );
+	RegisterNamespace ( kXMP_NS_PDFX_ID, "pdfxid" );
 	
-	XMP_StringPtr	voidPtr;
-	XMP_StringLen	voidLen;
-	
-	(void) RegisterNamespace ( kXMP_NS_XML, "xml", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_RDF, "rdf", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_DC, "dc", &voidPtr, &voidLen );
-
-	(void) RegisterNamespace ( kXMP_NS_XMP, "xap", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_PDF, "pdf", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_Photoshop, "photoshop", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_PSAlbum, "album", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_EXIF, "exif", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_EXIF_Aux, "aux", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_TIFF, "tiff", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_PNG, "png", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_JPEG, "jpeg", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_JP2K, "jp2k", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_CameraRaw, "crs", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_ASF, "asf", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_WAV, "wav", &voidPtr, &voidLen );
-
-	(void) RegisterNamespace ( kXMP_NS_AdobeStockPhoto, "bmsp", &voidPtr, &voidLen );
-
-	(void) RegisterNamespace ( kXMP_NS_XMP_Rights, "xapRights", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_XMP_MM, "xapMM", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_XMP_BJ, "xapBJ", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_XMP_Note, "xmpNote", &voidPtr, &voidLen );
-
-	(void) RegisterNamespace ( kXMP_NS_DM, "xmpDM", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_XMP_Text, "xapT", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_XMP_PagedFile, "xapTPg", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_XMP_Graphics, "xapG", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_XMP_Image, "xapGImg", &voidPtr, &voidLen );
-
-	(void) RegisterNamespace ( kXMP_NS_XMP_Font, "stFnt", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_XMP_Dimensions, "stDim", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_XMP_ResourceEvent, "stEvt", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_XMP_ResourceRef, "stRef", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_XMP_ST_Version, "stVer", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_XMP_ST_Job, "stJob", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_XMP_ManifestItem, "stMfs", &voidPtr, &voidLen );
-
-	(void) RegisterNamespace ( kXMP_NS_XMP_IdentifierQual, "xmpidq", &voidPtr, &voidLen );
-
-	(void) RegisterNamespace ( kXMP_NS_IPTCCore, "Iptc4xmpCore", &voidPtr, &voidLen );
-
-	(void) RegisterNamespace ( kXMP_NS_PDFA_Schema, "pdfaSchema", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_PDFA_Property, "pdfaProperty", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_PDFA_Type, "pdfaType", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_PDFA_Field, "pdfaField", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_PDFA_ID, "pdfaid", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_PDFA_Extension, "pdfaExtension", &voidPtr, &voidLen );
-
-	(void) RegisterNamespace ( kXMP_NS_PDFX, "pdfx", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( kXMP_NS_PDFX_ID, "pdfxid", &voidPtr, &voidLen );
-	
-	(void) RegisterNamespace ( "adobe:ns:meta/", "x", &voidPtr, &voidLen );
-	(void) RegisterNamespace ( "http://ns.adobe.com/iX/1.0/", "iX", &voidPtr, &voidLen );
+	RegisterNamespace ( "adobe:ns:meta/", "x" );
+	RegisterNamespace ( "http://ns.adobe.com/iX/1.0/", "iX" );
 
 // 06-Oct-07, ahu: Do not use aliases. They result in unexpected behaviour.
 //	XMPMeta::RegisterStandardAliases ( "" );
@@ -562,7 +731,6 @@ XMPMeta::Initialize()
 	
 	if ( ! XMPIterator::Initialize() ) XMP_Throw ( "Failure from XMPIterator::Initialize", kXMPErr_InternalFailure );
 	if ( ! XMPUtils::Initialize() ) XMP_Throw ( "Failure from XMPUtils::Initialize", kXMPErr_InternalFailure );
-
 	// Do miscelaneous semantic checks of types and arithmetic.
 
 	XMP_Assert ( sizeof(XMP_Int8) == 1 );
@@ -575,13 +743,14 @@ XMPMeta::Initialize()
 	XMP_Assert ( sizeof(XMP_Uns64) == 8 );
 	
 	XMP_Assert ( sizeof(XMP_OptionBits) == 4 );	// Check that option masking work on all 32 bits.
-	XMP_OptionBits flag = ~0UL;
+	XMP_OptionBits flag = ~0;
+
 	XMP_Assert ( flag == (XMP_OptionBits)(-1L) );
 	XMP_Assert ( (flag ^ kXMP_PropHasLang) == 0xFFFFFFBFUL );
 	XMP_Assert ( (flag & ~kXMP_PropHasLang) == 0xFFFFFFBFUL );
 	
 	XMP_OptionBits opt1 = 0;	// Check the general option bit macros.
-	XMP_OptionBits opt2 = ~0UL;
+	XMP_OptionBits opt2 = flag;
 	XMP_SetOption ( opt1, kXMP_PropValueIsArray );
 	XMP_ClearOption ( opt2, kXMP_PropValueIsArray );
 	XMP_Assert ( opt1 == ~opt2 );
@@ -652,7 +821,6 @@ XMPMeta::Terminate() RELEASE_NO_THROW
 	
 	XMPIterator::Terminate();
 	XMPUtils::Terminate();
-
 	EliminateGlobal ( sNamespaceURIToPrefixMap );
 	EliminateGlobal ( sNamespacePrefixToURIMap );
 	EliminateGlobal ( sRegisteredAliasMap );
@@ -663,7 +831,7 @@ XMPMeta::Terminate() RELEASE_NO_THROW
 	EliminateGlobal ( sExceptionMessage );
 
 	XMP_TermMutex ( sXMPCoreLock );
-
+	
 }	// Terminate
 
 
@@ -674,7 +842,7 @@ XMPMeta::Terminate() RELEASE_NO_THROW
 /* class-static */ void
 XMPMeta::Unlock ( XMP_OptionBits options )
 {
-	options = options;	// Avoid unused parameter warning.	// *** Need IgnoreParam macro.
+	UNUSED(options);
 
     #if TraceXMPLocking
     	fprintf ( xmpOut, "  Unlocking XMP toolkit, count = %d\n", sLockCount ); fflush ( xmpOut );
@@ -693,7 +861,7 @@ XMPMeta::Unlock ( XMP_OptionBits options )
 void
 XMPMeta::UnlockObject ( XMP_OptionBits options ) const
 {
-	options = options;	// Avoid unused parameter warning.
+	UNUSED(options);
 
 	XMPMeta::Unlock ( 0 );
 
@@ -713,7 +881,7 @@ XMPMeta::DumpNamespaces ( XMP_TextOutputProc outProc,
                           void *             refCon )
 {
 	XMP_Assert ( outProc != 0 );	// ! Enforced by wrapper.
-	XMP_Status status;
+	XMP_Status status = 0;
 	
 	XMP_StringMapPos p2uEnd = sNamespacePrefixToURIMap->end();	// ! Move up to avoid gcc complaints.
 	XMP_StringMapPos u2pEnd = sNamespaceURIToPrefixMap->end();
@@ -731,7 +899,7 @@ XMPMeta::DumpNamespaces ( XMP_TextOutputProc outProc,
 		XMP_StringMapPos nsOther = sNamespaceURIToPrefixMap->find ( nsLeft->second );
 		if ( (nsOther == u2pEnd) || (nsLeft != sNamespacePrefixToURIMap->find ( nsOther->second )) ) {
 			OutProcLiteral ( "  ** bad namespace URI **  " );
-			OutProcString ( nsLeft->second );
+			DumpClearString ( nsLeft->second, outProc, refCon );
 			goto FAILURE;
 		}
 		
@@ -739,7 +907,7 @@ XMPMeta::DumpNamespaces ( XMP_TextOutputProc outProc,
 			if ( nsRight == nsLeft ) continue;	// ! Can't start at nsLeft+1, no operator+!
 			if ( nsLeft->second == nsRight->second ) {
 				OutProcLiteral ( "  ** duplicate namespace URI **  " );
-				OutProcString ( nsLeft->second );
+				DumpClearString ( nsLeft->second, outProc, refCon );
 				goto FAILURE;
 			}
 		}
@@ -751,7 +919,7 @@ XMPMeta::DumpNamespaces ( XMP_TextOutputProc outProc,
 		XMP_StringMapPos nsOther = sNamespacePrefixToURIMap->find ( nsLeft->second );
 		if ( (nsOther == p2uEnd) || (nsLeft != sNamespaceURIToPrefixMap->find ( nsOther->second )) ) {
 			OutProcLiteral ( "  ** bad namespace prefix **  " );
-			OutProcString ( nsLeft->second );
+			DumpClearString ( nsLeft->second, outProc, refCon );
 			goto FAILURE;
 		}
 
@@ -759,7 +927,7 @@ XMPMeta::DumpNamespaces ( XMP_TextOutputProc outProc,
 			if ( nsRight == nsLeft ) continue;	// ! Can't start at nsLeft+1, no operator+!
 			if ( nsLeft->second == nsRight->second ) {
 				OutProcLiteral ( "  ** duplicate namespace prefix **  " );
-				OutProcString ( nsLeft->second );
+				DumpClearString ( nsLeft->second, outProc, refCon );
 				goto FAILURE;
 			}
 		}
@@ -787,7 +955,7 @@ XMPMeta::DumpAliases ( XMP_TextOutputProc outProc,
                        void *             refCon )
 {
 	XMP_Assert ( outProc != 0 );	// ! Enforced by wrapper.
-	XMP_Status status;
+	XMP_Status status = 0;
 
 	XMP_Assert ( sRegisteredAliasMap != 0 );
 
@@ -806,7 +974,7 @@ XMPMeta::DumpAliases ( XMP_TextOutputProc outProc,
 	for ( aliasPos = sRegisteredAliasMap->begin(); aliasPos != aliasEnd; ++aliasPos ) {
 
 		OutProcNChars ( "   ", 3 );
-		OutProcString ( aliasPos->first );
+		DumpClearString ( aliasPos->first, outProc, refCon );
 		OutProcPadding ( maxLen - aliasPos->first.size() );
 		OutProcNChars ( " => ", 4 );
 
@@ -855,11 +1023,10 @@ XMPMeta::GetGlobalOptions()
 // ----------------
 
 /* class-static */ void
-XMPMeta::SetGlobalOptions ( XMP_OptionBits options )
+XMPMeta::SetGlobalOptions ( XMP_OptionBits /*options*/ )
 {
 
 	XMP_Throw ( "Unimplemented method XMPMeta::SetGlobalOptions", kXMPErr_Unimplemented );
-		void * p; p = &options;	// Avoid unused param warnings.
 
 }	// SetGlobalOptions
 
@@ -868,60 +1035,22 @@ XMPMeta::SetGlobalOptions ( XMP_OptionBits options )
 // RegisterNamespace
 // -----------------
 
-/* class-static */ bool
+/* class-static */ void
 XMPMeta::RegisterNamespace ( XMP_StringPtr	 namespaceURI,
-							 XMP_StringPtr	 suggestedPrefix,
-							 XMP_StringPtr * registeredPrefix,
-							 XMP_StringLen * prefixSize )
+                             XMP_StringPtr	 prefix )
 {
-	bool	prefixMatches	= false;
-	
-	XMP_Assert ( (registeredPrefix != 0) && (prefixSize != 0) );		// ! Enforced by wrapper.
-	if ( (*namespaceURI == 0) || (*suggestedPrefix == 0) ) {
+	if ( (*namespaceURI == 0) || (*prefix == 0) ) {
 		XMP_Throw ( "Empty namespace URI or prefix", kXMPErr_BadParam );
 	}
 	
 	XMP_VarString	nsURI ( namespaceURI );
-	XMP_VarString	suggPrefix ( suggestedPrefix );
-	if ( suggPrefix[suggPrefix.size()-1] != ':' ) suggPrefix += ':';
-	VerifySimpleXMLName ( suggestedPrefix, suggestedPrefix+suggPrefix.size()-1 );	// Exclude the colon.
+	XMP_VarString	prfix ( prefix );
+	if ( prfix[prfix.size()-1] != ':' ) prfix += ':';
+	VerifySimpleXMLName ( prefix, prefix+prfix.size()-1 );	// Exclude the colon.
 	
-	XMP_StringMapPos	uriPos	= sNamespaceURIToPrefixMap->find ( nsURI );
-	
-	if ( uriPos == sNamespaceURIToPrefixMap->end() ) {
-		
-		// The URI is not yet registered, make sure we use a unique prefix.
-		
-		XMP_VarString	uniqPrefix ( suggPrefix );
-		int		suffix = 0;
-		char	buffer [32];
-
-		while ( true ) {
-			if ( sNamespacePrefixToURIMap->find ( uniqPrefix ) == sNamespacePrefixToURIMap->end() ) break;
-			++suffix;
-			snprintf ( buffer, sizeof(buffer), "_%d_:", suffix );	// AUDIT: Using sizeof for snprintf length is safe.
-			uniqPrefix = suggPrefix;
-			uniqPrefix.erase ( uniqPrefix.size()-1 );	// ! Remove the trailing ':'.
-			uniqPrefix += buffer;
-		}
-		
-		// Add the new namespace to both maps.
-		
-		XMP_StringPair	newNS ( nsURI, uniqPrefix );
-		uriPos = sNamespaceURIToPrefixMap->insert ( sNamespaceURIToPrefixMap->end(), newNS );
-		
-		newNS.first.swap ( newNS.second );
-		(void) sNamespacePrefixToURIMap->insert ( sNamespacePrefixToURIMap->end(), newNS );
-
-	}
-	
-	// Return the actual prefix and see if it matches the suggested prefix.
-	
-	*registeredPrefix = uriPos->second.c_str();
-	*prefixSize = uriPos->second.size();
-	
-	prefixMatches = ( uriPos->second == suggPrefix );
-	return prefixMatches;
+        // Set the new namespace in both maps.
+        (*sNamespaceURIToPrefixMap)[nsURI] = prfix;
+        (*sNamespacePrefixToURIMap)[prfix] = nsURI;
 	
 }	// RegisterNamespace
 
@@ -989,13 +1118,20 @@ XMPMeta::GetNamespaceURI ( XMP_StringPtr   namespacePrefix,
 // ---------------
 
 // *** Don't allow standard namespaces to be deleted.
+// *** We would be better off not having this. Instead, have local namespaces from parsing be
+// *** restricted to the object that introduced them.
 
 /* class-static */ void
 XMPMeta::DeleteNamespace ( XMP_StringPtr namespaceURI )
 {
-	namespaceURI = namespaceURI;	// Avoid unused parameter warning.
-	XMP_Assert ( *namespaceURI != 0 ); 	// ! Enforced by wrapper.
-	XMP_Throw ( "Unimplemented method XMPMeta::DeleteNamespace", kXMPErr_Unimplemented );   // *** #error "write me"
+	XMP_StringMapPos uriPos = sNamespaceURIToPrefixMap->find ( namespaceURI );
+	if ( uriPos == sNamespaceURIToPrefixMap->end() ) return;
+
+	XMP_StringMapPos prefixPos = sNamespacePrefixToURIMap->find ( uriPos->second );
+	if ( prefixPos == sNamespacePrefixToURIMap->end() ) return;
+	
+	sNamespaceURIToPrefixMap->erase ( uriPos );
+	sNamespacePrefixToURIMap->erase ( prefixPos );
 
 }	// DeleteNamespace
 
@@ -1195,13 +1331,12 @@ XMPMeta::ResolveAlias ( XMP_StringPtr	 aliasNS,
 // -----------
 
 /* class-static */ void
-XMPMeta::DeleteAlias ( XMP_StringPtr aliasNS,
-					   XMP_StringPtr aliasProp )
+XMPMeta::DeleteAlias ( XMP_StringPtr /*aliasNS*/,
+                       XMP_StringPtr /*aliasProp*/ )
 {
 
-	XMP_Assert ( (aliasNS != 0) && (aliasProp != 0) );	// Enforced by wrapper.
+//	Todo: XMP_Assert ( (aliasNS != 0) && (aliasProp != 0) );	/ / Enforced by wrapper.
 	XMP_Throw ( "Unimplemented method XMPMeta::DeleteAlias", kXMPErr_Unimplemented );   // *** #error "write me"
-		void * p; p = &aliasNS; p = &aliasProp;	// Avoid unused param warnings.
 
 }	// DeleteAlias
 
@@ -1290,10 +1425,10 @@ XMPMeta::DumpObject ( XMP_TextOutputProc outProc,
                       void *             refCon ) const
 {
 	XMP_Assert ( outProc != 0 );	// ! Enforced by wrapper.
-	XMP_Status status;
+	XMP_Status status = 0;
 	
 	OutProcLiteral ( "Dumping XMPMeta object \"" );
-	OutProcString ( tree.name );
+	DumpClearString ( tree.name, outProc, refCon );
 	OutProcNChars ( "\"  ", 3 );
 	status = DumpNodeOptions ( tree.options, outProc, refCon );
 	if ( status != 0 ) goto EXIT;
@@ -1305,7 +1440,7 @@ XMPMeta::DumpObject ( XMP_TextOutputProc outProc,
 	
 	if ( ! tree.value.empty() ) {
 		OutProcLiteral ( "** bad root value **  \"" );
-		OutProcString ( tree.value );
+		DumpClearString ( tree.value, outProc, refCon );
 		OutProcNChars ( "\"", 1 );
 		OutProcNewline();
 	}
@@ -1326,9 +1461,9 @@ XMPMeta::DumpObject ( XMP_TextOutputProc outProc,
 
 			OutProcNewline();
 			OutProcIndent ( 1 );
-			OutProcString ( currSchema->name );
+			DumpClearString ( currSchema->value, outProc, refCon );
 			OutProcNChars ( "  ", 2 );
-			OutProcString ( currSchema->value );
+			DumpClearString ( currSchema->name, outProc, refCon );
 			OutProcNChars ( "  ", 2 );
 			status = DumpNodeOptions ( currSchema->options, outProc, refCon );
 			if ( status != 0 ) goto EXIT;
@@ -1434,30 +1569,77 @@ XMPMeta::GetObjectOptions() const
 // ----------------
 
 void
-XMPMeta::SetObjectOptions ( XMP_OptionBits options )
+XMPMeta::SetObjectOptions ( XMP_OptionBits /*options*/ )
 {
 
 	XMP_Throw ( "Unimplemented method XMPMeta::SetObjectOptions", kXMPErr_Unimplemented );
-		void * p; p = &options;	// Avoid unused param warnings.
 
 }	// SetObjectOptions
 
 
+// -------------------------------------------------------------------------------------------------
+// Sort
+// ----
+//
+// At the top level the namespaces are sorted by their prefixes. Within a namespace, the top level
+// properties are sorted by name. Within a struct, the fields are sorted by their qualified name,
+// i.e. their XML prefix:local form. Unordered arrays of simple items are sorted by value. Language
+// Alternative arrays are sorted by the xml:lang qualifiers, with the "x-default" item placed first.
+
+void
+XMPMeta::Sort()
+{
+
+	if ( ! this->tree.qualifiers.empty() ) {
+		sort ( this->tree.qualifiers.begin(), this->tree.qualifiers.end(), CompareNodeNames );
+		SortWithinOffspring ( this->tree.qualifiers );
+	}
+
+	if ( ! this->tree.children.empty() ) {
+		// The schema prefixes are the node's value, the name is the URI, so we sort schemas by value.
+		sort ( this->tree.children.begin(), this->tree.children.end(), CompareNodeValues );
+		SortWithinOffspring ( this->tree.children );
+	}
+
+}	// Sort
+
+
+// -------------------------------------------------------------------------------------------------
+// Erase
+// -----
+//
+// Clear everything except for clientRefs.
+
+void
+XMPMeta::Erase()
+{
+
+	this->prevTkVer = 0;
+	if ( this->xmlParser != 0 ) {
+		delete ( this->xmlParser );
+		this->xmlParser = 0;
+	}
+	this->tree.ClearNode();
+
+}	// Erase
+
+
 // -------------------------------------------------------------------------------------------------
 // Clone
 // -----
 
-XMPMeta *
-XMPMeta::Clone ( XMP_OptionBits options ) const
+void
+XMPMeta::Clone ( XMPMeta * clone, XMP_OptionBits options ) const
 {
+	if ( clone == 0 ) XMP_Throw ( "Null clone pointer", kXMPErr_BadParam );
 	if ( options != 0 ) XMP_Throw ( "No options are defined yet", kXMPErr_BadOptions );
-	
-	XMPMeta * clone = new XMPMeta;
+	XMP_Assert ( this->tree.parent == 0 );
+
+	clone->tree.ClearNode();
 	
 	clone->tree.options = this->tree.options;
 	clone->tree.name    = this->tree.name;
 	clone->tree.value   = this->tree.value;
-	XMP_Assert ( this->tree.parent == 0 );
 
 	#if 0	// *** XMP_DebugBuild
 		clone->tree._namePtr = clone->tree.name.c_str();
@@ -1466,9 +1648,6 @@ XMPMeta::Clone ( XMP_OptionBits options ) const
 	
 	CloneOffspring ( &this->tree, &clone->tree );
 	
-	XMP_Assert ( clone->clientRefs == 0 );	// Gets incremneted later.
-	return clone;
-	
 }	// Clone
 
 // =================================================================================================
